/**
 * 
 */
package eu.etaxonomy.taxeditor.printpublisher.wizard;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.print.PublishConfigurator;
import eu.etaxonomy.cdm.print.Publisher;
import eu.etaxonomy.cdm.print.out.IPublishOutputModule;
import eu.etaxonomy.taxeditor.model.CdmProgressMonitorAdapter;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.printpublisher.internal.PrintpublisherPlugin;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>AbstractPublishWizard class.</p>
 *
 * @author n.hoffmann
 * @created Apr 1, 2010
 * @version 1.0
 */
public abstract class AbstractPublishWizard extends Wizard implements IExportWizard, IHasPersistableSettings{
			
	/** Constant <code>PAGE_SERVICE="PAGE_SERVICE"</code> */
	public static final String PAGE_SERVICE = "PAGE_SERVICE";
	/** Constant <code>PAGE_TAXA="PAGE_TAXA"</code> */
	public static final String PAGE_TAXA = "PAGE_TAXA";
	/** Constant <code>PAGE_OPTIONS="PAGE_OPTIONS"</code> */
	public static final String PAGE_OPTIONS = "PAGE_OPTIONS";
	/** Constant <code>PAGE_FOLDER="PAGE_FOLDER"</code> */
	public static final String PAGE_FOLDER = "PAGE_FOLDER";
	/** Constant <code>PAGE_OVERVIEW="PAGE_OVERVIEW"</code> */
	public static final String PAGE_OVERVIEW = "PAGE_OVERVIEW";
	/** Constant <code>PAGE_FEATURETREE="PAGE_FEATURETREE"</code> */
	public static final String PAGE_FEATURETREE = "PAGE_FEATURETREE";
	/** Constant <code>PAGE_STYLESHEET="PAGE_STYLESHEET"</code> */
	public static final String PAGE_STYLESHEET = "PAGE_STYLESHEET";
		
	protected SelectServiceWizardPage pageService;
	protected SelectTaxaWizardPage pageTaxa;
	protected SelectOptionsWizardPage pageOptions;
	protected SelectFeatureTreeWizardPage pageFeatureTree;
	protected SelectDirectoryWizardPage pageFolder;
	protected SelectStylesheetWizardPage pageStylesheet;

	private PublishConfigurator configurator;

	private IPublishOutputModule outputModule;

	/**
	 * <p>Constructor for AbstractPublishWizard.</p>
	 */
	public AbstractPublishWizard(){		
		setNeedsProgressMonitor(true);
		setDialogSettings(PrintpublisherPlugin.getDefault().getDialogSettings());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean performFinish() {
		
		Job job = new Job("Running Print Publisher") {

			@Override
			protected IStatus run(IProgressMonitor monitor) {
				monitor.beginTask("Print Publisher", getConfigurator().calculateNumberOfNodes() + 1);
				ConversationHolder conversation = null;
				try{					
					getConfigurator().setProgressMonitor(CdmProgressMonitorAdapter.CreateMonitor(monitor));
					
					if(getConfigurator().isLocal()){
						conversation = CdmStore.createConversation();
						// we want to enforce that the session is closed and nothing is 
						// instantiated beneath the regular cdmlib-remote object boundaries 
						conversation.commit(false);
					}
					
					Publisher.publish(getConfigurator());
					
				}finally{
					monitor.done();
					if(conversation != null) conversation.close();
					Display.getDefault().asyncExec(new Runnable() {
						@Override
						public void run() {
							MessageDialog.openInformation(Display.getDefault().getActiveShell(), "Print Publisher", "Your document was created.");
						}
					});
				}
				return Status.OK_STATUS;
			}
			
		};
		
		job.setPriority(Job.BUILD);
		job.schedule();
		
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public void addPages() {
		super.addPages();
		
		pageService = new SelectServiceWizardPage(PAGE_SERVICE);
		addPage(pageService);
				
		pageOptions = new SelectOptionsWizardPage(PAGE_OPTIONS);
		addPage(pageOptions);
		
		pageFeatureTree = new SelectFeatureTreeWizardPage(PAGE_FEATURETREE);
		addPage(pageFeatureTree);
		
		pageStylesheet = new SelectStylesheetWizardPage(PAGE_STYLESHEET);
		addPage(pageStylesheet);

		pageFolder = new SelectDirectoryWizardPage(PAGE_FOLDER);
		addPage(pageFolder);
		
	}
	
	@Override
	public void loadSettings() {
		for(IWizardPage page : getPages()){
			if(page instanceof IHasPersistableSettings){
				((IHasPersistableSettings) page).loadSettings();
			}
		}		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	/** {@inheritDoc} */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		MessagingUtils.info("Instantiating wizard: " + this.getClass().getSimpleName());
	}
	

	
	/** {@inheritDoc} */
	@Override
	public boolean canFinish() {
		boolean canFinish = true;
		
		canFinish &= pageService.isPageComplete(); 
		
		// persistable settings pages
		canFinish &= pageFeatureTree.isPageComplete();
		canFinish &= pageFolder.isPageComplete();
		canFinish &= pageStylesheet.isPageComplete();
		
		return canFinish;
	}
	
	/**
	 * <p>Getter for the field <code>configurator</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.print.PublishConfigurator} object.
	 */
	protected PublishConfigurator getConfigurator() {
		return configurator;
	}
	
	/**
	 * <p>Setter for the field <code>configurator</code>.</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.print.PublishConfigurator} object.
	 */
	protected void setConfigurator(PublishConfigurator configurator){
		this.configurator = configurator;
	}
	
	/**
	 * <p>Setter for the field <code>outputModule</code>.</p>
	 *
	 * @param outputModule a {@link eu.etaxonomy.cdm.print.out.IPublishOutputModule} object.
	 */
	protected void setOutputModule(IPublishOutputModule outputModule){
		this.outputModule = outputModule;
	}
	
	/**
	 * <p>Getter for the field <code>outputModule</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.print.out.IPublishOutputModule} object.
	 */
	public IPublishOutputModule getOutputModule(){
		return outputModule;
	}
}
