// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.printpublisher.wizard;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.jdom.Element;

import eu.etaxonomy.cdm.print.IXMLEntityFactory;
import eu.etaxonomy.cdm.print.XMLHelper;
import eu.etaxonomy.taxeditor.featuretree.FeatureTreeEditorWizard;

/**
 * <p>SelectFeatureTreeWizardPage class.</p>
 *
 * @author n.hoffmann
 * @created Aug 6, 2010
 * @version 1.0
 */
public class SelectFeatureTreeWizardPage extends AbstractPublishWizardPage 
	implements ISelectionChangedListener, IHasPersistableSettings{
	
	private static final String DIALOG_SETTING_FEATURE_TREE_UUID = "dialogSettingFeatureTreeUuid";
	private ListViewer viewer;

	/**
	 * <p>Constructor for SelectFeatureTreeWizardPage.</p>
	 *
	 * @param pageName a {@link java.lang.String} object.
	 */
	protected SelectFeatureTreeWizardPage(String pageName) {
		super(pageName);
		setTitle("Select Feature Tree");
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	/** {@inheritDoc} */
	@Override
	public void createControl(Composite parent) {
		setPageComplete(false);
		
		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());
		
		viewer = new ListViewer(composite);
		viewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		viewer.setContentProvider(new FeatureTreeContentProvider());
		viewer.setLabelProvider(new FeatureTreeLabelProvider());
		
		viewer.addSelectionChangedListener(this);
				
		// offer feature tree editor when local
		if(getConfigurator().isLocal()){
			final Button button_openFeatureTree = new Button(composite, SWT.PUSH);
			button_openFeatureTree.setText("Open FeatureTree Editor");
			
			button_openFeatureTree.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					FeatureTreeEditorWizard featureTreeEditorDialog = new FeatureTreeEditorWizard();
					WizardDialog dialog = new WizardDialog(getShell(), featureTreeEditorDialog);
					
					if(dialog.open() == IStatus.OK){
						refresh();
					}
				}
			});
		}
				
		// when using a local factory we can set the input right here
		if(getConfigurator() != null && getConfigurator().isLocal()){
			refresh();
		}
		
		loadSettings();
		
		setControl(composite);
	}

	@Override
	public void loadSettings() {
		String featureTreeUuidString = getDialogSettingValue(DIALOG_SETTING_FEATURE_TREE_UUID); 
		
		if(featureTreeUuidString != null){
			setSelectedFeatureTree(featureTreeUuidString);
			
		}
	}
	
	/** {@inheritDoc} */
	@Override
	public void selectionChanged(SelectionChangedEvent event) {
		IStructuredSelection selection = (IStructuredSelection) event.getSelection();
		
		if(selection.isEmpty()){
			return;
		}
		
		setPageComplete(true);
		
		
		Element featureTreeElement = (Element) selection.getFirstElement();
			
		UUID featureTreeUuid = XMLHelper.getUuid(featureTreeElement);
		
		setFeatureTree(featureTreeUuid.toString());
	}
	
	private void setSelectedFeatureTree(String featureTreeUuid){
		List<Element> input = (ArrayList<Element>) viewer.getInput();
		
		for(Element element : input){
			UUID uuid = XMLHelper.getUuid(element);
			if(uuid.toString().equals(featureTreeUuid)){
				viewer.setSelection(new StructuredSelection(element));
				break;
			}
		}
	}
	
	public void setFeatureTree(String featureTreeUuidString){
		putDialogSettingValue(DIALOG_SETTING_FEATURE_TREE_UUID, featureTreeUuidString);
		UUID featureTreeUuid = UUID.fromString(featureTreeUuidString);
		getConfigurator().setFeatureTree(featureTreeUuid);
		setPageComplete(featureTreeUuid != null);
	}
	
	/** {@inheritDoc} */
	@Override
	public boolean canFlipToNextPage() {
		return isPageComplete();
	}

	/**
	 * Refreshes the input of the viewer
	 */
	public void refresh() {
		IXMLEntityFactory factory = getConfigurator().getFactory();

		List<Element> featureTrees = factory.getFeatureTrees();
		List<Element> realFeatureTrees = new ArrayList<Element>();
		for(Element featureTree : featureTrees){
			if(XMLHelper.getClazz(featureTree).equals("FeatureTree")){
				realFeatureTrees.add(featureTree);
			}
		}
		viewer.setInput(realFeatureTrees);
	}
	
	private class FeatureTreeContentProvider implements IStructuredContentProvider {

		@Override
		public void dispose() {}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}

		@Override
		public Object[] getElements(Object inputElement) {
			if(inputElement instanceof List){
				return ((List) inputElement).toArray();
			}
			return new Object[0];
		}
		
	}
	
	private class FeatureTreeLabelProvider extends LabelProvider {
		
		/*
		 * (non-Javadoc)
		 * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
		 */
		@Override
		public String getText(Object element) {
			if(element instanceof Element){
				Element xmlElement = (Element) element;			
				return XMLHelper.getTitleCache(xmlElement);
			}
			return "no title cache";
		}
		
	}
}
